SuperCollider3

SC3 is an object oriented language based on Smalltalk, 
open source, general programming environment, 
specially suited for audio (and multimedia).

		The Two Worlds in SC3:
		
SClang			-		SCServer
(Lang, client)

text / code		- 		sound
discrete			-		continous
numbers			-		signals
data				-		buffer
scheduling		-		envelopes
objects			-		unit generators 
messages			-		patching

[connection : OSC = OpenSoundControl = UDP protocol]

Open Main help file: 
OSX - Cmd-D, Win - F1, emacs - C-c C-h


Handling: 
text oriented interpreter, 
executing lines of code 

			// interpret line or highlighted area (cmd-enter)
1 + 2;	
[0, 2, 3, 4] + 3;
			// everything is an object and understands messages
1.dump; 	
[0, 2, 3, 4].dump;

[0, 2, 3, 4] + [10, 20];
4.squared;
[0, 2, 3, 4].squared;
[[0, 2], [3, 4]].squared;

	// minimal pattern preview
a = Pseq([0, 2, 3]);	// a pattern is a template for 
							// generating a stream of values
b = a.asStream;			// 
b.next;
b.next;
b.next;
b.next;

b = (a + 10).asStream; 
4.do { b.next.postln; }
s = Server.local.boot;
	// use a pattern for making a series of events
Pbind(\note, a).trace.play;
Pbind(\note, a + 12).trace.play;
Pbind(\note, a + [0,0.3,0.5,0.7].rand2.squared).trace.play;
Pbind(\note, a.squared).trace.play;


			// braces allow to select easy and determine scopes
(
(1.5 + 2.25).postln;	// add .postln here
10 + 20;
4567;
)	


	Math operations (see Operators.help)
	// lists (arrays) are objects as well
[ 1, 2, 3, 4 ].squared;
[ 1, 2, [3, 4] ].squared;	// lists can be in lists

	// precedence: unary messages first, then 
	// binary math operations strictly left to right
[1, 2, 3] + 100 * [4, 5, 6];	
[1, 2, 3] + (100 * [4, 5, 6]);	

	// syntax variants:
objA + objB		// binary operation, math style
objA.min(objB) 	// binary operation, message style
objA min: objB 	// binary operation, keyword style

	// unary operations
objA.squared 
objA.sin	

Variables:

	// local variables only exist within scope
(
var otto; 
otto = 123; 
otto.postln;
otto = [123, 234,345];
otto.postln;
otto = "i am a string...";
)

otto = 6; 		// outside of scope: error!


	// global variables: single letters a-z.
	// syntax: variableName = object.message(argument);
a = [1, 2, 3, 5].min(3);

a * 100;

	// environment variables: access by ~<name> 
	// better naming, good for keeping things around
~myList = [1, 2, 3, 5];
~myList * 10;


	// dictionaries: keeping things by name
~dict = (); 	// empty
~dict[\myList] = [1,2,3,5];

	// nesting dictionaries - organizing things by name
~dict[\myLists] = (); 
~dict[\myLists][\notes1] = [1,2,3,5];
~dict[\myLists][\notes2] = [1,2,3,5].reverse + 2;

~dict[\myList]

	// syntax variant: message-like style
~dict.myList = [0, 2, 3];
~dict.myList_([0, 2, 3]);
	// access
~dict.myList * 10;


Functions: 
			// defined once, they can be executed several times
f = { 10.rand };
f.value;
			// polymorphism
f = { 10.0.rand };
f.value;
			// calculate with functions
g = f + 100;
g.value;


Some important help files 
- General 

Help
How-to-Use-the-Interpreter
More-On-Getting-Help
Intro-to-Objects	

- Synthesis: 
Tour_of_UGens
UGens-and-Synths

Randomness

// Just In Time Programming, Proxies
JITLib
NodeProxy
ProxySpace


Some recommended tutorials:
Julio dEscrivan (bitbongo.com), very simple.
David Cottle, Computer Music with SC3 (by mail from him)
d<dot>cottle<at>utah<dot>edu


Signals: 
			// synthesis is done at the server with UGens
		 	// a lot of unit generators, see Tour_of_UGens 
UGen.allSubclasses.size;
UGen.allSubclasses.collect(_.name).printAll;
	

s.boot	;
s.makeWindow;
s.makeGui;


			// function becomes a synthdef, 
			// which is compiled and sent to the server
{ SinOsc.ar * 0.1 }.play;	

			// stop synthesis with Cmd-. (or alt-. or ...)
	
{ SinOsc.ar(600) * 0.2 }.scope;

			// calculate with UGens: adding is mixing 
{ SinOsc.ar(300) + Dust.ar(400) * 0.1 }.scope;

			// arrays become audio channels
{ [SinOsc.ar(400) * 2, Dust.ar(400), 0, 0, SinOsc.ar( 800 ), SinOsc.ar( 1000 )] * 0.1 }.scope;

			// change values while running
x = { arg freq=200; Impulse.ar(freq).lag(0.001) * 1 }.play;
x.set(\freq, 250);
x.set(\freq, 500);

			// tasks: time scheduling on the client
(
Task({
	100.do({ 
		x.set(\freq, 300 + 400.rand);
		0.2.wait;
	});
}).play;
)
x.play;