---- Routines and Tasks----

Routines and Tasks allow you to create functions that can wait for a while and then resume execution:

(
// using the SynthDef from a class; we now use the method "store" to write the SynthDef to disk and put it in the library of the server, so we can use it with the Events we learn about later on
SynthDef( \simplesine, { arg out=0, amp=0.05, freq=220, t_trig=1, dur=1, fm=50;
	Out.ar( out, 
		EnvGen.kr( Env.perc, t_trig, timeScale: dur, doneAction: 2 ) *
		Mix.new( SinOsc.ar( freq*[1,1.5,1.7,2] + SinOsc.ar( fm, 0, 20 ), 0, amp ) )
	) 
} ).store;
)

a = [1, 1.5, 1.7, 2];

(
r = Routine.new( {
	a.do{ arg item, index;
		[index, item].postln;
		1.0.wait; // wait for a second
	};
	"done".postln;
});
)

r.play;


( // making sound:
r = Routine.new( {
	a.do{ arg item, index;
		Synth.new( \simplesine, [\freq, item*300] );
		[index, item].postln;
		2.0.wait;
	};
	"done".postln;
});
)

r.play;

/// Similar example with a Task

( // making sound:
t = Task.new( {
	a.do{ arg item, index;
		Synth.new( \simplesine, [\freq, item*300] );
		[index, item].postln;
		2.0.wait;
	};
	"done".postln;
});
)

t.play;
// we can pause a Task:
t.pause;
t.play;
t.stop;
// if the Task has ended and we play it again it does not work:
t.play;
// because it needs to be reset:
t.reset;
t.play;


------ Patterns --------

With a pattern:

( // making sound:
p = Pbind(
	\instrument,\simplesine,
	\freq, Pseq( a )*300,
	\dur, 1.0,
	\legato, 2
//	\delta, 2.0
)
);
p.play;




Pbind(\freq, 600).play

TempoClock

Pbind(\freq, Pfunc({rrand(100, 900)})).play;

Pbind(
	\freq, Pfunc({rrand(100, 900)}),
	\dur, Pfunc({rrand(0.1, 1.5)})).play

Pbind(
	\degree, Pfunc({8.rand}),
	\oct, Pfunc({rrand(3, 7)}), //or try \octave?
	\dur, 0.2).play

Pbind(
	\scale, [0, 2, 4, 6, 8, 10],
	\degree, Pfunc({6.rand}), 
	\oct, Pfunc({rrand(3, 7)}), //or try \octave
	\dur, 0.2).play

Pbind(
	\scale, [0, 2, 3, 5, 6, 8, 10, 11],
	\degree, Pfunc({8.rand}), 
	\oct, Pfunc({rrand(3, 7)}), //or try \octave
	\dur, 0.2).play


---------------


An excellent guide to patterns, exists in the help files,
"A Practical Guide to Patterns"

Some of the main material to read through:

[PG_01_Introduction]
[PG_02_Basic_Vocabulary]
[PG_03_What_Is_Pbind]
[PG_04_Words_to_Phrases]
[PG_05_Math_on_Patterns]

[PG_07_Value_Conversions]

[PG_Cookbook01_Basic_Sequencing]

Look at Pdef too

TabFileReader
File
String

UsingMIDI